/*************************************************************
 *  File Name:  READASCI.C
 *
 *  Purpose:  Reads an ascii image file. Data values should be
 *	separated by spaces or a comma.
 *
 *  Passed Parameters:
 *	filename:    name of the file containing the ascii data
 *	image_ptr:   pointer to the image to fill with this data
 *
 *  Returned Parameters:
 *	min:	     minimum value (>= 0) in the input image
 *	max:         maximum value in the input image
 *
 *  Programmer:  Barbara Marks
 *
 *  Date:  7 December 1992
 **************************************************************/
#include <stdio.h>
#include <string.h>
#include "stats.h"


void read_ascii (filename,image_ptr,min,max)
char	*filename;
short   *image_ptr;
short	*min,*max;
{
	int	i,j;
	int	val;
	short	*ptr;
	FILE	*fp;
	

	if ((fp = fopen(filename,"r")) == NULL) {
		printf("\nERROR reading file: %s\n",filename);
		exit(-1);
	}

	*min = 9999;
	*max = -9999;
	MAX_CLASSES = -999;
	land_border = FALSE;
	ptr = image_ptr;

/*
 *  total_size is defined in the include file stats.h.  total_size
 *  is the number of cells inside the landscape (non-background and
 *  non-landscape border cells).
 */
	total_size = 0;

	
	for (i=0; i < num_rows; i++) {
  	   for (j=0; j < num_cols; j++) {
	        fscanf (fp,"%d",&val);

	        *ptr = (short) val;
/*
 *  The value for background cells (cells outside the area of
 *  interest) was input by the user.  Change these to -999
 *  to make life easier in other routines.
 */
		if (*ptr == background) *ptr = -999;

/*
 *  If there are class values < 0 and not background, then this
 *  image includes a landscape border.  Set a flag.
 */
	 	if (*ptr < 0 && *ptr != -999) land_border = TRUE;

/*
 *  Find the minimum and maximum class values (only consider those
 *  classes inside the landscape).  Also set flags for each class 
 *  that exists (inside the lanscape).
 */
	   	if (*ptr < *min && *ptr >= 0) *min = *ptr;
		if (*ptr > *max) *max = *ptr;
		if (*ptr >= 0) total_size ++;

/*
 *  If this image has a landscape border, for contagion the total
 *  number of classes is needed including those class types in the
 *  landscape border.
 */
	        if (*ptr >= 0 && *ptr > MAX_CLASSES)
		   MAX_CLASSES = *ptr;
	        if (*ptr < 0 && *ptr != -999 && -(*ptr) > MAX_CLASSES)
                   MAX_CLASSES = -(*ptr);

		ptr++;
	   }
	   fscanf (fp,"\n");
	}

	fclose (fp);
	MAX_CLASSES ++;
}
